import logging
import inquirer
from pathlib import Path
from dotenv import set_key, dotenv_values
from typing import List, Tuple
from ant_upload_checker.directory_selector import DirectorySelector


ENV_FILE_PATH = Path(".env").resolve()


def setup_logging() -> None:
    logging.basicConfig(level=logging.INFO, format="%(message)s")


def check_if_user_wants_to_override_env_file() -> bool:
    question = [
        inquirer.List(
            "choice",
            message="Existing user settings were found, do you want to overwrite this?",
            choices=["Yes", "No"],
            default="No",
        )
    ]
    response = inquirer.prompt(question)

    return response["choice"] == "Yes"


def save_user_info_to_env() -> None:
    if not ENV_FILE_PATH.is_file() or check_if_user_wants_to_override_env_file():
        get_user_info_api_key()
        get_user_info_directories()
        logging.info("User setting saved to '.env' file")


def get_user_info_directories():
    selector = DirectorySelector()
    input_folders, output_folder = selector.run_directory_selector()
    input_directories_str = ",".join(input_folders)

    set_key(
        dotenv_path=ENV_FILE_PATH,
        key_to_set="INPUT_FOLDERS",
        value_to_set=input_directories_str,
    )
    set_key(
        dotenv_path=ENV_FILE_PATH,
        key_to_set="OUTPUT_FOLDER",
        value_to_set=output_folder,
    )


def get_user_info_api_key() -> None:
    prompt = [inquirer.Text("api_key", message="Please enter your API key from ANT")]

    user_api_key = inquirer.prompt(prompt)["api_key"]

    if user_api_key:
        set_key(
            dotenv_path=ENV_FILE_PATH,
            key_to_set="API_KEY",
            value_to_set=user_api_key,
        )
    else:
        raise ValueError("No API key was set - please restart")


def load_env_file() -> Tuple[str, list[Path], Path]:
    settings = dotenv_values(ENV_FILE_PATH)

    expected_settings = ["API_KEY", "INPUT_FOLDERS", "OUTPUT_FOLDER"]
    missing_settings = [
        setting for setting in expected_settings if settings.get(setting) is None
    ]

    if missing_settings:
        raise ValueError(
            f"The following user settings were not found in the '.env' file: {','.join(missing_settings)}"
            "\nPlease restart"
        )

    api_key = settings["API_KEY"]

    input_folders_str = settings["INPUT_FOLDERS"].split(",")
    input_folders = [Path(folder) for folder in input_folders_str]

    output_folder = Path(settings["OUTPUT_FOLDER"])

    logging.info("User settings loaded from '.env' file\n")

    return api_key, input_folders, output_folder



import pandas as pd
import numpy as np
from rebulk.match import MatchesDict
import logging
from guessit import guessit
from pathlib import Path
import re
import sys
import shutil
from typing import List, Dict


class FilmProcessor:
    def __init__(self, input_folders: list[str], output_folder: str):
        self.file_extensions: list[str] = ["mp4", "avi", "mkv", "mpeg", "m2ts"]
        self.input_folders: list[str] = input_folders
        self.output_folder: Path = Path(output_folder)
        self.film_list_df_types: dict[str, str] = {
            "Full file path": "string",
            "Parsed film title": "string",
            "Film size (GB)": "float64",
            "Resolution": "string",
            "Codec": "string",
            "Source": "string",
            "Release group": "string",
            "Already on ANT?": "string",
            "Info": "string",
        }
        self.csv_file_path: Path = self.output_folder / "Film list.csv"
        self.backup_csv_file_path: Path = (
            self.output_folder / "Film list old version backup.csv"
        )

    def get_film_file_paths(self) -> list[Path]:
        """
        Scan the input folder from parameters for the given file extensions,
        adding any files to a list.
        """
        paths = []
        for ext in self.file_extensions:
            for folder in self.input_folders:
                paths.extend(Path(folder).glob(f"**/*.{ext}"))

        filtered_paths = self.remove_paths_containing_extras_folder(paths)
        cleaned_paths = self.remove_paths_if_unopenable(filtered_paths)

        if not cleaned_paths:
            raise ValueError(
                "No films were found, check the INPUT_FOLDERS value in parameters.py"
            )

        cleaned_paths.sort()

        return cleaned_paths

    def get_film_info_from_file_paths(
        self, film_file_paths: list[Path]
    ) -> pd.DataFrame:
        film_sizes = self.get_film_sizes_from_file_paths(film_file_paths)
        guessed_films = self.get_guessit_info_from_film_paths(film_file_paths)
        film_titles = self.get_formatted_titles_from_guessed_films(guessed_films)
        film_resolutions = self.get_film_resolutions_from_guessed_films(guessed_films)
        film_codecs = self.get_codec_from_guessed_films(guessed_films)
        film_sources = self.get_source_from_guessed_films(guessed_films)
        release_groups = self.get_release_groups_from_guessed_films(guessed_films)

        film_list_df = self.create_film_list_dataframe(
            film_file_paths,
            film_sizes,
            film_titles,
            film_resolutions,
            film_codecs,
            film_sources,
            release_groups,
        )

        return film_list_df

    def combine_with_existing_film_csv(
        self, film_list_df: pd.DataFrame
    ) -> pd.DataFrame:
        should_read_csv = self.check_if_existing_film_csv_exists()

        if not should_read_csv:
            return film_list_df

        existing_film_list = pd.read_csv(self.csv_file_path)

        should_combine_film_lists = self.check_if_existing_csv_is_compatible(
            existing_film_list
        )

        if should_combine_film_lists:
            combined_film_list = self.combine_current_film_list_with_existing_csv(
                existing_film_list, film_list_df
            )

            return combined_film_list

        return film_list_df

    def combine_current_film_list_with_existing_csv(
        self, existing_film_list: pd.DataFrame, current_film_list: pd.DataFrame
    ) -> pd.DataFrame:

        logging.info(
            "Combining existing output file with current list of films "
            "and dropping duplicate film titles..."
        )
        existing_film_list_formatted = existing_film_list.astype(
            self.film_list_df_types
        )

        combined_film_list = (
            pd.concat([current_film_list, existing_film_list_formatted])
            .drop_duplicates(subset=["Parsed film title"], keep="last")
            .reset_index(drop=True)
            .fillna("")  # Fill NAs with empty strings for later dupe handling
        )

        self.stop_process_if_all_films_already_in_existing_csv(combined_film_list)

        return combined_film_list

    def stop_process_if_all_films_already_in_existing_csv(
        self, combined_film_list_df: pd.DataFrame
    ) -> None:
        if all(
            combined_film_list_df["Already on ANT?"].str.contains(
                r"^Duplicate:", regex=True, na=False
            )
        ):
            logging.info(
                "All films have already been searched and are duplicates. "
                "\n\nEnding the process early.\n\n----"
            )
            sys.exit(0)

    def remove_paths_containing_extras_folder(
        self, file_paths: list[Path]
    ) -> list[Path]:
        """
        Remove any file file paths containing Extras as their parent folder.
        """
        cleaned_paths = [path for path in file_paths if path.parent.name != "Extras"]

        return cleaned_paths

    def get_film_sizes_from_file_paths(self, file_paths: list[Path]) -> list[float]:
        film_sizes = [self.get_file_size_from_path(path) for path in file_paths]

        return film_sizes

    def remove_paths_if_unopenable(self, paths):
        """
        If file does not exist or is not openable, remove from paths.
        Warn user if path exceeds 260 characters.
        """
        cleaned_paths = []
        for path in paths:
            if not path.is_file():
                file_name = path.stem
                warning_message = (
                    f"{file_name} could not be opened or does not exist, skipping."
                )
                if len(str(path)) > 260:
                    warning_message += " This may be caused by a file path exceeding 260 characters. Try shortening the folder or file name."
                logging.warning(warning_message)
            else:
                cleaned_paths.append(path)

        return cleaned_paths

    def get_file_size_from_path(self, file_path: Path) -> float:
        file_size_in_bytes = file_path.stat().st_size
        file_size = self.convert_bytes_to_gb(file_size_in_bytes)

        return file_size

    def convert_bytes_to_gb(self, num_in_bytes: int) -> float:
        """
        Convert bytes to correct unit of measurement as a string
        """
        num_in_gb = round((num_in_bytes / 1073741824), 2)

        return num_in_gb

    def get_guessit_info_from_film_paths(
        self, file_paths: list[Path]
    ) -> list[MatchesDict]:
        """
        Use guessit package to extract film information
        into ordered dictionary.
        """
        guessed_films = [guessit(path) for path in file_paths]

        return guessed_films

    def get_formatted_titles_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        """
        Get film titles from guessit objects, then fix titles missing
        full stops within acronyms
        """
        titles = [
            self.get_film_attribute_from_guessed_film(film, "title")
            for film in guessed_films
        ]
        cleaned_titles = [self.fix_title_if_contains_acronym(title) for title in titles]

        return cleaned_titles

    def get_film_attribute_from_guessed_film(
        self, guessed_film: MatchesDict, attribute: str
    ) -> str:
        """
        Extract the given guessit attribute from a given guessit object.
        """
        try:
            film_attribute = guessed_film[attribute]
            if isinstance(film_attribute, list):
                film_attribute = ", ".join(film_attribute)
        except:
            film_attribute = ""

        return film_attribute

    def get_film_resolutions_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_resolutions = [
            self.get_film_attribute_from_guessed_film(film, "screen_size")
            for film in guessed_films
        ]

        return film_resolutions

    def get_codec_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_codecs = [
            self.get_film_attribute_from_guessed_film(film, "video_codec")
            for film in guessed_films
        ]

        film_codecs_cleaned = [re.sub(r"\.", "", codec) for codec in film_codecs]

        return film_codecs_cleaned

    def get_source_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        film_sources = [
            self.get_film_attribute_from_guessed_film(film, "source")
            for film in guessed_films
        ]

        film_sources_cleaned = [
            re.sub("Ultra HD Blu-ray", "Blu-ray", source) for source in film_sources
        ]

        return film_sources_cleaned

    def get_release_groups_from_guessed_films(
        self, guessed_films: list[MatchesDict]
    ) -> list[str]:
        release_groups = [
            self.get_film_attribute_from_guessed_film(film, "release_group").lower()
            for film in guessed_films
        ]

        return release_groups

    def fix_title_if_contains_acronym(self, film_title: str) -> str:
        """
        After guessit has extracted film title, fix instances where
        consecutive single letter words contain spaces instead of full stops.
        e.g. L A Confidential -> L.A Confidential -> L.A. Confidential
        e.g. S W A T -> S.W.A.T -> S.W.A.T.
        """
        acronym_spaces_as_full_stops = re.sub(
            r"(?<=\b[A-Za-z]{1})\s(?=[A-Za-z]{1}\b)", ".", film_title
        )

        acronym_suffixed_with_a_full_stop = re.sub(
            r"(?<=\.[A-Za-z])(\s|$)(?=[^\s])", ". ", acronym_spaces_as_full_stops
        )

        acronym_at_end_of_title_suffixed_with_full_stop = re.sub(
            r"(?<=\..$)",
            ".",
            acronym_suffixed_with_a_full_stop,
        )

        return acronym_at_end_of_title_suffixed_with_full_stop

    def create_film_list_dataframe(
        self,
        film_file_paths: list[Path],
        film_sizes: list[float],
        film_titles: list[str],
        film_resolutions: list[str],
        film_codecs: list[str],
        film_sources: list[str],
        film_release_groups: list[str],
    ) -> pd.DataFrame:
        """
        Combine the full file paths and film titles into a
        pandas DataFrame.
        Add empty ANT check column before we attempt to combine an existing
        film list csv, in case one does not exist.
        """
        films_df = pd.DataFrame(
            {
                "Full file path": film_file_paths,
                "Parsed film title": film_titles,
                "Film size (GB)": film_sizes,
                "Resolution": film_resolutions,
                "Codec": film_codecs,
                "Source": film_sources,
                "Release group": film_release_groups,
                "Already on ANT?": np.repeat("", len(film_file_paths)),
                "Info": np.repeat("", len(film_file_paths)),
            }
        ).astype(self.film_list_df_types)

        return films_df

    def check_if_existing_film_csv_exists(self) -> bool:
        if not self.csv_file_path.is_file():
            logging.info(
                "An existing output file (%s) was not found, processing films "
                "from scratch...",
                self.csv_file_path,
            )
            return False

        logging.info("An existing output file (%s) was found.", self.csv_file_path)

        return True

    def check_if_existing_csv_is_compatible(
        self, existing_film_df: pd.DataFrame
    ) -> bool:
        existing_columns = list(existing_film_df.columns)

        if existing_columns != list(self.film_list_df_types.keys()):
            logging.warning(
                "Warning: existing file was created using an old version of ANT upload checker.\n"
                "-------- Existing file is being skipped as it doesn't contain all the required columns.\n"
                "-------- Creating a backup film list from the previous version (%s).\n"
                "-------- This can be deleted if you don't need it.\n",
                self.backup_csv_file_path,
            )

            shutil.copy(self.csv_file_path, self.backup_csv_file_path)

            return False

        return True



from typing import Any, Union
from pathlib import Path
import pandas as pd
from ant_upload_checker import constants


class DupeChecker:
    def __init__(self, films_to_dupe_check: pd.DataFrame):
        self.films_to_dupe_check: pd.DataFrame = films_to_dupe_check
        self.guid_missing_message: str = "(Failed to extract URL from API response)"
        self.not_found_message: tuple[str, str] = (
            "Uploadable - potentially",
            "Film not found on ANT - does not already exist, or title failed to match",
        )
        self.banned_groups = constants.BANNED_GROUPS

    def check_if_films_can_be_uploaded(self) -> pd.DataFrame:
        dupe_checked_films = self.films_to_dupe_check.copy()

        films_to_skip = dupe_checked_films.loc[dupe_checked_films["Should skip"]]
        films_to_dupe_check = dupe_checked_films.loc[
            ~dupe_checked_films["Should skip"]
        ].copy()

        films_to_dupe_check[["Already on ANT?", "Info"]] = films_to_dupe_check.apply(
            lambda film: self.check_if_film_is_duplicate(
                film["Full file path"],
                film["Resolution"],
                film["Codec"],
                film["Source"],
                film["Release group"],
                film["API response"],
            ),
            axis=1,
            result_type="expand",
        )

        combined_films = (
            pd.concat([films_to_skip, films_to_dupe_check])
            .drop(["Should skip", "API response"], axis=1)
            .sort_values(
                by=["Already on ANT?", "Parsed film title"], ascending=[False, True]
            )
        )

        return combined_films

    def check_if_film_is_duplicate(
        self,
        full_file_path: str,
        resolution: str,
        codec: str,
        source: str,
        release_group: str,
        api_response: list[dict[str, Any]],
    ) -> str:
        if not api_response:
            return self.not_found_message

        # Retain lower() to account for previous film list versions
        if release_group.lower() in self.banned_groups:
            return (
                "Banned",
                f"Release group '{release_group}' is banned from ANT - do not upload",
            )

        file_name = Path(full_file_path).name

        filename_info_if_match = self.check_if_filename_exists_on_ant(
            file_name, api_response
        )
        if filename_info_if_match is not None:
            return filename_info_if_match

        dupe_properties = {"resolution": resolution, "codec": codec, "media": source}
        return self.check_remaining_dupe_properties(dupe_properties, api_response)

    def check_if_filename_exists_on_ant(
        self, file_name: str, api_response: list[dict[str, Any]]
    ) -> Union[str, None]:
        for existing_upload in api_response:
            uploaded_files = existing_upload.get("files", [])

            if uploaded_files:
                for file_info in uploaded_files:
                    if file_name == file_info.get("name", ""):
                        return (
                            "Duplicate",
                            f"Exact filename already exists: {existing_upload.get('guid',self.guid_missing_message)}",
                        )

    def check_remaining_dupe_properties(
        self,
        dupe_properties,
        api_response: list[dict[str, Any]],
    ) -> str:
        missing_properties = [k for k, v in dupe_properties.items() if not v]
        available_properties = {k: v for k, v in dupe_properties.items() if v}

        if len(missing_properties) == len(dupe_properties):
            return (
                "Duplicate - potentially",
                f"On ANT, but could not dupe check (could not extract {'/'.join(missing_properties)} from filename). "
                f"{api_response[0].get('guid',self.guid_missing_message)}",
            )

        return self.perform_dupe_check(
            available_properties, missing_properties, api_response
        )

    def perform_dupe_check(
        self,
        available_properties: dict[str, str],
        missing_properties: list[str],
        api_response: list[dict[str, Any]],
    ) -> str:
        """
        With the available properties, check if the film is a full duplicate, partial duplicate,
        or not a duplicate.
        """

        available_properties_str = "/".join(available_properties.values())
        existing_guid = self.guid_missing_message

        for existing_upload in api_response:
            existing_guid = existing_upload.get("guid", self.guid_missing_message)

            is_duplicate = all(
                available_properties[prop].lower()
                == existing_upload.get(prop, "").lower()
                for prop in available_properties
            )
            if is_duplicate:
                dupe_string = f"A film with {available_properties_str} already exists"
                if missing_properties:
                    return (
                        "Duplicate - partial",
                        f"{dupe_string}. Could not extract and check "
                        f"{'/'.join(missing_properties)} from filename. {existing_guid}",
                    )
                else:
                    return ("Duplicate", f"{dupe_string}: {existing_guid}")

        return (
            "Uploadable",
            f"A film with {available_properties_str} does not already exist. {existing_guid}",
        )



import pandas as pd
import numpy as np
import pytest
import logging
from ant_upload_checker.film_searcher import FilmSearcher

LOGGER = logging.getLogger(__name__)


@pytest.fixture
def return_mock_search_for_film_on_ant_not_found(monkeypatch):
    def mockreturn(test_arg, test_arg_2):
        return []

    monkeypatch.setattr(
        "test_film_searcher.FilmSearcher.search_for_film_title_on_ant", mockreturn
    )


@pytest.fixture
def return_mock_search_for_film_on_ant_torrentid(monkeypatch):
    def mockreturn(test_arg, test_arg_2):
        return [{"guid": "url/torrentid=1"}]

    monkeypatch.setattr(
        "test_film_searcher.FilmSearcher.search_for_film_title_on_ant", mockreturn
    )


def test_check_if_films_exist_on_ant(
    return_mock_search_for_film_on_ant_not_found, caplog
):
    # pre dupe check, torrentid links would be skipped from searching
    # post 1.7.0, films are only skipped from searching
    # if start with Duplicate:
    # (as these may have since been uploaded by others)

    caplog.set_level(logging.INFO)

    test_df = pd.DataFrame(
        {
            "Full file path": [
                "C:/Movies/Test (2020)/Test (2020).mkv",
                "C:/Movies/Another film (2020)/Another film (2020).mkv",
                "C:/Movies/Test film (2020)/Test film (2020).mkv",
                "C:/Movies/New film (2020)/New film (2020).mkv",
                "C:/Movies/An awesome test film 1080p H264 Web-DL (2020).mkv",
                "C:/Movies/Batman Begins 2160p H265 Blu-ray (2005).mkv",
                "C:/Movies/zebras",
            ],
            "Film size (GB)": [1.11, 1.22, 1.33, 1.44, 2.0, 5.0, 1.0],
            "Parsed film title": [
                "Test",
                "Another film",
                "test film",
                "New film",
                "An awesome test film",
                "Batman Begins",
                "zebras",
            ],
            "Resolution": ["1080p", "1080p", "1080p", "1080p", "1080p", "2160p", ""],
            "Codec": ["H264", "H264", "H264", "H264", "H264", "H265", ""],
            "Source": ["Blu-ray", "Blu-ray", "Blu-ray", "Web", "Web", "Blu-ray", ""],
            "Already on ANT?": [
                "Resolution already uploaded: link/torrentid=1",  # Pre 1.7.0, will be re-searched post 1.7.0
                "NOT FOUND",
                np.nan,
                np.nan,
                "Duplicate: exact filename already exists: test_link",
                "Partial duplicate: test_link",
                "x is banned from ANT - do no not upload",
            ],
        }
    )

    fs = FilmSearcher(test_df, "test_api_key")
    actual_df = fs.check_if_films_exist_on_ant()
    expected_df = pd.DataFrame(
        {
            "Full file path": [
                "C:/Movies/An awesome test film 1080p H264 Web-DL (2020).mkv",
                "C:/Movies/Another film (2020)/Another film (2020).mkv",
                "C:/Movies/Batman Begins 2160p H265 Blu-ray (2005).mkv",
                "C:/Movies/New film (2020)/New film (2020).mkv",
                "C:/Movies/Test (2020)/Test (2020).mkv",
                "C:/Movies/Test film (2020)/Test film (2020).mkv",
                "C:/Movies/zebras",
            ],
            "Film size (GB)": [2.0, 1.22, 5.0, 1.44, 1.11, 1.33,1.0],
            "Parsed film title": [
                "An awesome test film",
                "Another film",
                "Batman Begins",
                "New film",
                "Test",
                "test film",
                "zebras",
            ],
            "Resolution": ["1080p", "1080p", "2160p", "1080p", "1080p", "1080p", ""],
            "Codec": ["H264", "H264", "H265", "H264", "H264", "H264", ""],
            "Source": ["Web", "Blu-ray", "Blu-ray", "Web", "Blu-ray", "Blu-ray", ""],
            "Already on ANT?": [
                "Duplicate: exact filename already exists: test_link",
                "NOT FOUND",
                "Partial duplicate: test_link",
                np.nan,
                "Resolution already uploaded: link/torrentid=1",
                np.nan,
                "x is banned from ANT - do no not upload",
            ],
            "Should skip": [True, False, True, False, False, False, True],
            "API response": [[], [], [], [], [], [], []],
        }
    )

    assert (
        "Skipping 3 films already found on ANT in the previous output file..."
        in caplog.text
    )
    pd.testing.assert_frame_equal(actual_df, expected_df)


@pytest.mark.parametrize("film_title", ["A film", "A film with and in the title"])
def test_check_if_film_exists_on_ant_false(
    return_mock_search_for_film_on_ant_not_found, caplog, film_title
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.check_if_film_exists_on_ant(film_title)

    assert f"Searching for {film_title}" in caplog.text
    assert actual_return == []
    assert "--- Not found on ANT ---" in caplog.text

    if "and" in film_title:
        assert "Searching for A film with & in the title as well" in caplog.text


@pytest.mark.parametrize("film_title", ["A film"])
def test_check_if_film_exists_on_ant_true(
    return_mock_search_for_film_on_ant_torrentid, caplog, film_title
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.check_if_film_exists_on_ant(film_title)

    assert f"Searching for {film_title}" in caplog.text
    assert actual_return == [{"guid": "url/torrentid=1"}]
    assert "--- Not found on ANT ---" not in caplog.text


def test_replace_word_and_re_search(
    return_mock_search_for_film_on_ant_not_found, caplog
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    test_film = "Test film biscuit"
    test_regex = "biscuit"
    test_replacement = "jaffa"

    actual_return = fs.replace_word_and_re_search(
        test_film, test_regex, test_replacement
    )

    assert actual_return == []
    assert "Searching for Test film jaffa as well..." in caplog.text


films_four_numbers = {
    "1208 East of Bucharest": "Searching for 12:08 East of Bucharest as well...",
    "Test film 1508": "Searching for Test film 15:08 as well...",
    "Film 1000 test": "Searching for Film 10:00 test as well...",
}


@pytest.mark.parametrize("test_film", films_four_numbers)
def test_search_for_film_if_contains_potential_time_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="time"
    )

    assert actual_return == []
    expected_log_info = films_four_numbers[test_film]
    assert expected_log_info in caplog.text
    assert "Film title may contain a date or time" in caplog.text


films_five_numbers = ["12345 test film", "Test film 12345", "Film 12345 test"]


@pytest.mark.parametrize("test_film", films_five_numbers)
def test_search_for_film_if_contains_potential_time_false(
    return_mock_search_for_film_on_ant_torrentid, caplog, test_film
):
    # Film search mock return is the torrentid. This should never be returned
    # If the film does not contain 4 numbers, as the search is skipped.
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="time"
    )

    assert actual_return == []
    assert "Film title may contain a date or time" not in caplog.text


films_with_dates_numbers = {
    "77 One Day in London": "Searching for 7/7 One Day in London as well...",
    "Test film 11": "Searching for Test film 1/1 as well...",
    "Film 89 test": "Searching for Film 8/9 test as well...",
    "911 film": "Searching for 9/11 film as well...",
    "Fahrenheit 911": "Searching for Fahrenheit 9/11 as well...",
    "Film 112 test": "Searching for Film 1/12 test as well...",
}


@pytest.mark.parametrize("test_film", films_with_dates_numbers)
def test_search_for_film_if_contains_potential_date_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)
    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="date"
    )

    assert actual_return == []
    expected_log_info = films_with_dates_numbers[test_film]
    assert expected_log_info in caplog.text
    assert "Film title may contain a date or time" in caplog.text


films_with_numbers_not_dates = [
    "7777 One Day in London",
    "Test film 1112",
    "Film 8910 test",
    "9 film",
    "Fahrenheit 91144",
    "Film 112444 test",
]


@pytest.mark.parametrize("test_film", films_five_numbers)
def test_search_for_film_if_contains_potential_date_false(
    return_mock_search_for_film_on_ant_torrentid, caplog, test_film
):
    # Film search mock return is the torrentid. This should never be returned
    # If the film does not contain 4 numbers, as the search is skipped.
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")

    actual_return = fs.search_for_film_if_contains_potential_date_or_time(
        test_film, format="date"
    )

    assert actual_return == []
    assert "Film title may contain a date or time" not in caplog.text


films_with_alternate_titles = {
    "Alphaville une etrange aventure de Lemmy Caution AKA Alphaville": (
        "Alphaville une etrange aventure de Lemmy Caution",
        "Alphaville",
    ),
    "Title 1 aka Title 2": ("Title 1", "Title 2"),
}


@pytest.mark.parametrize("test_film", films_with_alternate_titles)
def test_search_for_film_if_contains_aka_true(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.search_for_film_if_contains_aka(test_film)

    assert actual_return == []
    assert "Film title may contain an alternate title" in caplog.text
    assert (
        f"Searching for {films_with_alternate_titles[test_film][0]} as well..."
        in caplog.text
    )
    assert (
        f"Searching for {films_with_alternate_titles[test_film][1]} as well..."
        in caplog.text
    )


films_with_no_alternate_title = [
    "Aka Test film",
    "Test film aka",
    "Film with baka in it",
]


@pytest.mark.parametrize("test_film", films_with_no_alternate_title)
def test_search_for_film_if_contains_aka_false(
    return_mock_search_for_film_on_ant_not_found, caplog, test_film
):
    caplog.set_level(logging.INFO)

    fs = FilmSearcher("test", "test_api_key")
    actual_return = fs.search_for_film_if_contains_aka(test_film)

    assert actual_return == []
    assert "Film title may contain an alternate title" not in caplog.text
    assert f"Searching for Test film as well" not in caplog.text



BANNED_GROUPS = [
    "evo",
    "3lton",
    "4yeo",
    "ade",
    "afg",
    "anihls",
    "animerg",
    "aniurl",
    "aroma",
    "axxo",
    "brrip",
    "chd",
    "cm8",
    "crewsade",
    "d3g",
    "ddr",
    "dnl",
    "deadfish",
    "dnl",
    "elite",
    "esc",
    "fangding0",
    "fgt",
    "frds",
    "fum",
    "haiku",
    "hd2dvd",
    "hds",
    "hdtime",
    "hi10",
    "ion10",
    "iplanet",
    "jive",
    "kingdom",
    "leffe",
    "ligas",
    "load",
    "megusta",
    "mkvcage",
    "mhd",
    "msd",
    "nhanc3",
    "nhd",
    "noivtc",
    "nsd",
    "oj",
    "ozlem",
    "pirates",
    "prodji",
    "rapidcows",
    "rarbg",
    "retropeeps",
    "rdn",
    "resurrection",
    "rmteam",
    "santi",
    "sicfoi",
    "spasm",
    "spdvd",
    "stuttershit",
    "tbs",
    "telly",
    "tm",
    "upinsmoke",
    "uranime",
    "waf",
    "xred",
    "xs",
    "yify",
    "yts",
    "zeus",
    "zkbl",
    "zmn",
    "zmnt",
]



import tkinter as tk
from tkinter import filedialog
import logging


class DirectorySelector:
    def __init__(self) -> None:
        self.root = tk.Tk()
        self.root.protocol("WM_DELETE_WINDOW", self.root.quit)

    def add_folder(self) -> None:
        """
        Open a dialog to select a given directory, add it to the listbox
        """
        folder = filedialog.askdirectory(title="Select folder containing films")
        if folder:
            current_folder_selection = list(self.folder_listbox.get(0, tk.END))
            if folder in current_folder_selection:
                logging.info("Folder has already been selected, skipping")
            else:
                self.folder_listbox.insert(tk.END, folder)

    def select_input_directories(self) -> list[str]:
        """
        Provide a UI for the user to select multiple input directories to be scanned
        """
        logging.info(
            "In the dialog box, please select one or more directories containing your films"
        )

        self.root.deiconify()
        self.folder_listbox = tk.Listbox(
            self.root,
            selectmode=tk.SINGLE,
            width=100,
            height=15,
        )

        self.root.title("Select one or more input directories containing your films")
        self.folder_listbox.pack(pady=20)

        add_button = tk.Button(self.root, text="Add folder", command=self.add_folder)
        add_button.pack(pady=10)

        save_button = tk.Button(
            self.root, text="Finish folder selection", command=self.root.quit
        )
        save_button.pack(pady=10)
        self.root.mainloop()

        input_directories = list(self.folder_listbox.get(0, tk.END))

        if not input_directories:
            raise ValueError(
                "No folders were selected in the dialog box, please re-run"
            )

        # Hide but don't close
        self.root.withdraw()

        return input_directories

    def select_output_directory(self) -> str:
        """
        Provide a UI for the user to select a single output directory
        where the film list csv will later be saved.
        """
        self.root.withdraw()
        logging.info("In the dialog box, please select your output directory")
        output_directory = filedialog.askdirectory(
            title="Please select your output directory",
        )

        if not output_directory:
            raise ValueError(
                "No output folder was selected in the dialog box, please re-run"
            )

        return output_directory

    def run_directory_selector(self) -> tuple[list[str], str]:
        """
        Prompt user to select multiple input directories, then a single output directory,
        then close the Tkinter root window
        """
        input_directories = self.select_input_directories()
        output_directory = self.select_output_directory()

        self.root.destroy()

        return input_directories, output_directory



from ant_upload_checker.dupe_checker import DupeChecker
import pandas as pd
import pytest


test_values = [
    ("file_path", "1080p", "H264", "Blu-ray", "group", []),
    (
        "C:/films/test_film_name.mkv",
        "1080p",
        "H264",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [{"size": "100", "name": "test_film_name.mkv"}],
            }
        ],
    ),
    (
        "C:/films/test_film_name_2.mkv",
        "1080p",
        "H264",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link_2",
                "files": [
                    {"size": "1", "name": "unwanted_file.nfo"},
                    {"size": "100", "name": "test_film_name_2.mkv"},
                ],
            }
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "720p",
        "H264",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_filmy.mkv"},
                ],
                "resolution": "720p",
                "codec": "H265",
                "media": "Blu-ray",
            },
            {  # duplicate resolution, codec, source
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_different_name.mkv"},
                ],
                "resolution": "720p",
                "codec": "H264",
                "media": "Blu-ray",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "2160p",
        "H265",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "2160p",
                "codec": "H265",
                "media": "BLU-RAY",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "2160p",
        "H265",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "2160p",
                "codec": "H264",  # not a dupe, as codec differs
                "media": "Blu-ray",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "1080p",
        "H264",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "2160p",  # not a dupe, as resolution differs
                "codec": "H264",
                "media": "Blu-ray",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "",  # missing resolution from guessed film
        "H264",
        "Blu-ray",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "1080p",
                "codec": "H264",
                "media": "Blu-ray",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "1080p",
        "",  # missing codec from guessed film
        "",  # missing source from guessed film
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "1080p",
                "codec": "H264",
                "media": "Blu-ray",
            },
        ],
    ),
    (
        "C:/films/testing_film.mkv",
        "",  # missing
        "",  # missing codec from guessed film
        "Web",
        "group",
        [
            {
                "guid": "test_link",
                "files": [
                    {"size": "1", "name": "testing_film_non_exact_filename_match.mkv"},
                ],
                "resolution": "1080p",
                "codec": "H264",
                "media": "WEB",
            },
        ],
    ),
    (
        "C:/films/test_film_name.mkv",
        "1080p",
        "H264",
        "Blu-ray",
        "KiNGDOM",
        [
            {
                "guid": "test_link",
                "files": [{"size": "100", "name": "test_film_name.mkv"}],
            }
        ],
    ),
    (
        "C:/films/test_film_name.mkv",
        "",
        "",
        "",
        "test",
        [
            {
                "guid": "test_link",
                "files": [{"size": "100", "name": "test_film_name_x.mkv"}],
            }
        ],
    ),
]
expected_values = [
    (
        "Uploadable - potentially",
        "Film not found on ANT - does not already exist, or title failed to match",
    ),
    ("Duplicate", "Exact filename already exists: test_link"),
    ("Duplicate", "Exact filename already exists: test_link_2"),
    ("Duplicate", "A film with 720p/H264/Blu-ray already exists: test_link"),
    ("Duplicate", "A film with 2160p/H265/Blu-ray already exists: test_link"),
    ("Uploadable", "A film with 2160p/H265/Blu-ray does not already exist. test_link"),
    ("Uploadable", "A film with 1080p/H264/Blu-ray does not already exist. test_link"),
    (
        "Duplicate - partial",
        "A film with H264/Blu-ray already exists. Could not extract and check resolution from filename. test_link",
    ),
    (
        "Duplicate - partial",
        "A film with 1080p already exists. Could not extract and check codec/media from filename. test_link",
    ),
    (
        "Duplicate - partial",
        "A film with Web already exists. Could not extract and check resolution/codec from filename. test_link",
    ),
    ("Banned", "Release group 'KiNGDOM' is banned from ANT - do not upload"),
    (
        "Duplicate - potentially",
        "On ANT, but could not dupe check (could not extract resolution/codec/media from filename). test_link",
    ),
]


@pytest.mark.parametrize(
    ("test_input", "test_output"),
    zip(test_values, expected_values),
)
def test_check_if_film_is_duplicate(test_input, test_output):
    dupe_checker = DupeChecker(pd.DataFrame())

    actual_return = dupe_checker.check_if_film_is_duplicate(
        test_input[0],
        test_input[1],
        test_input[2],
        test_input[3],
        test_input[4],
        test_input[5],
    )

    assert actual_return == test_output



import pytest
import logging
from pathlib import Path
from ant_upload_checker.film_processor import FilmProcessor
import pandas as pd
import numpy as np
from collections import OrderedDict

LOGGER = logging.getLogger(__name__)


def test_get_film_file_paths(tmp_path):
    temp_mp4_file = tmp_path / "test.mp4"
    temp_mp4_file.write_text("Test")

    temp_extras_directory = tmp_path / "Extras"
    temp_extras_directory.mkdir()
    temp_mp4_file_to_be_removed = tmp_path / "Extras" / "test_2.mp4"
    temp_mp4_file_to_be_removed.write_text("Test")

    fp = FilmProcessor(input_folders=[tmp_path.parent], output_folder="")

    actual_output = fp.get_film_file_paths()
    expected_output = [temp_mp4_file]

    assert actual_output == expected_output


@pytest.fixture
def test_extras_file_paths():
    test_paths = [
        r"C:/The Extras 2030/The Extras 2030.mkv",
        r"C:/The Extras 2030/Extras/Bloopers.mkv",
        r"C:/Godzilla: King of the Monsters (2019)/Godzilla: King of the Monsters (2019).mkv",
        r"C:/Godzilla: King of the Monsters (2019)/Extras/Godzilla extras (2019).mkv",
        r"C:/Godzilla: King of the Monsters (2019)/Extras/Godzilla: King of the Monsters (2019).mkv",
    ]
    test_paths = [Path(x) for x in test_paths]
    return test_paths


def test_remove_paths_containing_extras_folder(test_extras_file_paths):
    fp = FilmProcessor("test", "test")
    actual_list = fp.remove_paths_containing_extras_folder(test_extras_file_paths)

    expected_list = [
        r"C:/The Extras 2030/The Extras 2030.mkv",
        r"C:/Godzilla: King of the Monsters (2019)/Godzilla: King of the Monsters (2019).mkv",
    ]
    expected_list = [Path(x) for x in expected_list]

    assert actual_list == expected_list


@pytest.fixture
def test_film_paths():
    test_list = [
        r"C:/Atlantics (2019)/Atlantique.2019.2160p.mkv",
        r"C:/tick tick... BOOM! 2021 test.720p.mkv",
        r"C:/Da.5.Bloods.2020..1080p.test.mkv",
        r"C:/Short term 12 2013/Short term 12 2013.1080p.film info.mkv",
        r"C:/X: First Class (2100).1080p.mkv",
        r"C:/Nick Fury: Agent of S.H.I.E.L.D. (1998).1080p.mkv",
        r"C:/L.A. Confidential (1997) .1080p.mkv",
        r"C:/A.I. Artificial Intelligence (2100).1080p.mkv",
        r"C:/G.I. Jane (2100).mkv",
        r"C:/E.T. the Extra-Terrestrial (2100).1080p.mkv",
        r"C:/S.W.A.T. (2100).1080p.mkv",
        r"C:/T.E.S. Test film (2010).1080p.mkv",
        r"C:/T.E.S.T. Test film (2010).1080p.mkv",
    ]
    test_list = [Path(x) for x in test_list]

    return test_list


@pytest.fixture
def test_guessit_films():
    test_guessit_films = [
        {
            "title": "Atlantics",
            "screen_size": "2160p",
            "source": "Ultra HD Blu-ray",
            "video_codec": "H.265",
            "release_group": "TEST",
        },
        {
            "title": "tick tick BOOM!",
            "screen_size": "720p",
            "source": "DVD",
            "video_codec": "MPEG-2",
            "release_group": "TEST",
        },
        {
            "title": "Da 5 Bloods",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "Short term 12",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "X: First Class",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "Nick Fury: Agent of S.H.I.E.L.D.",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "L A Confidential",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "A I Artificial Intelligence",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "G I Jane",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "E T the Extra-Terrestrial",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "S.W.A.T.",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
            "release_group": "TEST",
        },
        {
            "title": "T E S Test film",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
        },
        {
            "title": "T E.S.T Test film",
            "screen_size": "1080p",
            "source": "Blu-ray",
            "video_codec": "H.264",
        },
        {
            "title": "Test film",
            "screen_size": "1080p",
            "source": ["Blu-ray", "DVD"],
            "video_codec": "H.264",
        },
    ]

    ordered_dict_guessit_films = [OrderedDict(x) for x in test_guessit_films]

    return ordered_dict_guessit_films


def test_get_guessit_info_from_film_paths(test_film_paths):
    fp = FilmProcessor("test", "test")
    actual_guessit_films = fp.get_guessit_info_from_film_paths(test_film_paths)

    expected_type = OrderedDict
    assert all([isinstance(x, expected_type) for x in actual_guessit_films])


def test_get_film_attribute_from_guessed_film(test_guessit_films):
    fp = FilmProcessor("test", "test")

    actual_list = [
        fp.get_film_attribute_from_guessed_film(x, "title") for x in test_guessit_films
    ]
    expected_list = [
        "Atlantics",
        "tick tick BOOM!",
        "Da 5 Bloods",
        "Short term 12",
        "X: First Class",
        "Nick Fury: Agent of S.H.I.E.L.D.",
        "L A Confidential",
        "A I Artificial Intelligence",
        "G I Jane",
        "E T the Extra-Terrestrial",
        "S.W.A.T.",
        "T E S Test film",
        "T E.S.T Test film",
        "Test film",
    ]

    assert actual_list == expected_list


def test_get_source_film_attribute_from_guessed_film(test_guessit_films):
    fp = FilmProcessor("test", "test")

    actual_list = fp.get_source_from_guessed_films(test_guessit_films)

    expected_list = [
        "Blu-ray",  # Ultra-HD blu ray gets converted into blu-ray
        "DVD",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray",
        "Blu-ray, DVD",  # list of sources gets put into string
    ]

    assert actual_list == expected_list


def test_get_codec_film_attribute_from_guessed_films(test_guessit_films):
    fp = FilmProcessor("test", "test")

    actual_list = fp.get_codec_from_guessed_films(test_guessit_films)

    expected_list = [
        "H265",
        "MPEG-2",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
        "H264",
    ]

    assert actual_list == expected_list


def test_get_release_group_from_guessed_films(test_guessit_films):
    fp = FilmProcessor("test", "test")

    actual_list = fp.get_release_groups_from_guessed_films(test_guessit_films)

    expected_list = [
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "test",
        "",
        "",
        "",
    ]

    assert actual_list == expected_list


def test_fix_title_if_contains_acronym():
    fp = FilmProcessor("test", "test")

    test_list = [
        "Atlantics",
        "tick tick BOOM!",
        "Da 5 Bloods",
        "Short term 12",
        "X: First Class",
        "Nick Fury: Agent of S.H.I.E.L.D.",
        "L A Confidential",
        "A I Artificial Intelligence",
        "G I Jane",
        "E T the Extra-Terrestrial",
        "S.W.A.T.",
        "T E S Test film",
        "T E.S.T Test film",
    ]

    actual_list = [fp.fix_title_if_contains_acronym(x) for x in test_list]

    expected_list = [
        "Atlantics",
        "tick tick BOOM!",
        "Da 5 Bloods",
        "Short term 12",
        "X: First Class",
        "Nick Fury: Agent of S.H.I.E.L.D.",
        "L.A. Confidential",
        "A.I. Artificial Intelligence",
        "G.I. Jane",
        "E.T. the Extra-Terrestrial",
        "S.W.A.T.",
        "T.E.S. Test film",
        "T.E.S.T. Test film",
    ]

    assert actual_list == expected_list


def test_get_formatted_titles_from_guessed_films(test_guessit_films):
    fp = FilmProcessor("test", "test")
    actual_list = fp.get_formatted_titles_from_guessed_films(test_guessit_films)

    expected_list = [
        "Atlantics",
        "tick tick BOOM!",
        "Da 5 Bloods",
        "Short term 12",
        "X: First Class",
        "Nick Fury: Agent of S.H.I.E.L.D.",
        "L.A. Confidential",
        "A.I. Artificial Intelligence",
        "G.I. Jane",
        "E.T. the Extra-Terrestrial",
        "S.W.A.T.",
        "T.E.S. Test film",
        "T.E.S.T. Test film",
        "Test film",
    ]

    assert actual_list == expected_list


def test_get_film_resolutions_from_guessed_film(test_guessit_films):
    fp = FilmProcessor("test", "test")
    actual_list = fp.get_film_resolutions_from_guessed_films(test_guessit_films)
    expected_list = [
        "2160p",
        "720p",
        "1080p",
        "1080p",
        "1080p",
        "1080p",
        "1080p",
        "1080p",
        "",  # File name for G.I. Jane has no resolution - expect ""
        "1080p",
        "1080p",
        "1080p",
        "1080p",
        "1080p",
    ]

    assert actual_list == expected_list


def test_create_film_list_dataframe():
    fp = FilmProcessor("test", "test")
    test_film_paths = [
        Path(r"C:\X: First Class (2100)"),
        Path(r"C:\Short term 12 2013\Short term 12 2013 film info.mkv"),
    ]
    test_film_titles = ["X: First Class", "Short term 12"]
    test_film_sizes = [5.11, 2.14]
    test_film_codecs = ["", ""]
    test_film_sources = ["Blu-ray", "Web"]
    test_film_resolutions = ["1080p", "1080p"]
    test_release_groups = ["", ""]

    actual_df = fp.create_film_list_dataframe(
        test_film_paths,
        test_film_sizes,
        test_film_titles,
        test_film_resolutions,
        test_film_codecs,
        test_film_sources,
        test_release_groups,
    )

    expected_df = pd.DataFrame(
        {
            "Full file path": [
                r"C:\X: First Class (2100)",
                r"C:\Short term 12 2013\Short term 12 2013 film info.mkv",
            ],
            "Parsed film title": ["X: First Class", "Short term 12"],
            "Film size (GB)": [5.11, 2.14],
            "Resolution": ["1080p", "1080p"],
            "Codec": ["", ""],
            "Source": ["Blu-ray", "Web"],
            "Release group": ["", ""],
            "Already on ANT?": ["", ""],
            "Info": ["", ""],
        }
    ).astype(
        {
            "Full file path": "string",
            "Parsed film title": "string",
            "Film size (GB)": "float64",
            "Resolution": "string",
            "Codec": "string",
            "Source": "string",
            "Release group": "string",
            "Already on ANT?": "string",
            "Info": "string",
        }
    )

    pd.testing.assert_frame_equal(actual_df, expected_df)


def test_convert_bytes_to_gb():
    fp = FilmProcessor("test", "test")
    test_list = [1073741824, 1610612736, 1309965025]
    actual_list = [fp.convert_bytes_to_gb(num) for num in test_list]
    expected_list = [1, 1.5, 1.22]

    assert actual_list == expected_list


def test_combine_current_film_list_with_existing_csv(tmp_path, caplog):
    caplog.set_level(logging.INFO)

    expected_dtypes = {
        "Full file path": "string",
        "Parsed film title": "string",
        "Film size (GB)": "float64",
        "Resolution": "string",
        "Codec": "string",
        "Source": "string",
        "Release group": "string",
        "Already on ANT?": "string",
        "Info": "string",
    }

    test_existing_film_df = pd.DataFrame(
        {
            "Full file path": ["test_path"],
            "Parsed film title": ["test"],
            "Film size (GB)": [10.4],
            "Resolution": ["test"],
            "Codec": ["test"],
            "Source": ["test"],
            "Release group": ["test"],
            "Already on ANT?": ["NOT FOUND"],
            "Info": [""],
        }
    )
    test_existing_film_df.to_csv(tmp_path.joinpath("Film list.csv"), index=False)

    test_current_film_list = pd.DataFrame(
        {
            "Full file path": ["test_path", "New film"],
            "Parsed film title": ["test", "New film"],
            "Film size (GB)": [10.4, 9.9],
            "Resolution": ["test", "test"],
            "Codec": ["test", "test"],
            "Source": ["test", "test"],
            "Release group": ["test", "test"],
            "Already on ANT?": ["", ""],
            "Info": ["", ""],
        }
    ).astype(expected_dtypes)

    fp = FilmProcessor(input_folders="", output_folder=tmp_path)

    actual_df = fp.combine_current_film_list_with_existing_csv(
        test_existing_film_df, test_current_film_list
    )

    expected_df = pd.DataFrame(
        {
            "Full file path": [
                "New film",
                "test_path",
            ],
            "Parsed film title": ["New film", "test"],
            "Film size (GB)": [9.9, 10.4],
            "Resolution": ["test", "test"],
            "Codec": ["test", "test"],
            "Source": ["test", "test"],
            "Release group": ["test", "test"],
            "Already on ANT?": [
                "",
                "NOT FOUND",
            ],  # Note first np.nan was filled with empty string
            "Info": ["", ""],
        }
    ).astype(expected_dtypes)

    pd.testing.assert_frame_equal(actual_df, expected_df)


def test_check_if_existing_film_csv_exists(tmp_path, caplog):
    caplog.set_level(logging.INFO)

    fp = FilmProcessor(input_folders="", output_folder=tmp_path)

    test_existing_film_df = pd.DataFrame(
        {
            "Full file path": [pd.NA],
            "Parsed film title": [pd.NA],
            "Film size (GB)": [pd.NA],
            "Codec": [pd.NA],
            "Source": [pd.NA],
            "Release group": [pd.NA],
            "Already on ANT?": [pd.NA],
        }
    )
    path_name = tmp_path.joinpath("Film list.csv")
    test_existing_film_df.to_csv(path_name, index=False)

    assert fp.check_if_existing_film_csv_exists() is True
    assert f"An existing output file ({path_name}) was found" in caplog.text


def test_false_check_if_existing_film_csv_exists(tmp_path, caplog):
    caplog.set_level(logging.INFO)

    fp = FilmProcessor(input_folders="", output_folder=tmp_path)
    path_name = tmp_path.joinpath("Film list.csv")

    assert fp.check_if_existing_film_csv_exists() is False
    assert (
        f"An existing output file ({path_name}) was not found, processing films from scratch..."
        in caplog.text
    )


def test_false_check_if_existing_csv_is_compatible(tmp_path, caplog):
    caplog.set_level(logging.INFO)

    fp = FilmProcessor(input_folders="", output_folder=tmp_path)

    # Resolution is missing from existing film list
    test_existing_film_df = pd.DataFrame(
        {
            "Full file path": [pd.NA],
            "Parsed film title": [pd.NA],
            "Film size (GB)": [pd.NA],
            "Codec": [pd.NA],
            "Source": [pd.NA],
            "Release group": [pd.NA],
            "Already on ANT?": [pd.NA],
        }
    )
    test_existing_film_df.to_csv(tmp_path.joinpath("Film list.csv"), index=False)

    assert fp.check_if_existing_csv_is_compatible(test_existing_film_df) is False
    assert (
        "Warning: existing file was created using an old version of ANT upload checker"
        in caplog.text
    )
    assert Path.is_file(tmp_path.joinpath("Film list old version backup.csv"))


def test_true_check_if_existing_csv_is_compatible(tmp_path, caplog):
    caplog.set_level(logging.INFO)
    test_existing_film_df = pd.DataFrame(
        {
            "Full file path": [pd.NA],
            "Parsed film title": [pd.NA],
            "Film size (GB)": [pd.NA],
            "Resolution": [pd.NA],
            "Codec": [pd.NA],
            "Source": [pd.NA],
            "Release group": [pd.NA],
            "Already on ANT?": [pd.NA],
            "Info": [pd.NA],
        }
    )
    test_existing_film_df.to_csv(tmp_path.joinpath("Film list.csv"), index=False)

    fp = FilmProcessor(input_folders="", output_folder=tmp_path)

    assert fp.check_if_existing_csv_is_compatible(test_existing_film_df) is True
    assert not Path.is_file(tmp_path.joinpath("Film list old version backup.csv"))



import logging
from ant_upload_checker import setup_functions
from ant_upload_checker.film_processor import FilmProcessor
from ant_upload_checker.film_searcher import FilmSearcher
from ant_upload_checker.output import write_film_list_to_csv
from ant_upload_checker.dupe_checker import DupeChecker


def main():
    setup_functions.setup_logging()
    logging.info("Starting ANT upload checker...")

    setup_functions.save_user_info_to_env()
    api_key, input_folders, output_folder = setup_functions.load_env_file()

    films = FilmProcessor(input_folders, output_folder)
    film_file_paths = films.get_film_file_paths()
    film_list_df = films.get_film_info_from_file_paths(film_file_paths)

    film_list_combined = films.combine_with_existing_film_csv(film_list_df)

    film_searcher = FilmSearcher(film_list_combined, api_key)
    films_to_dupe_check = film_searcher.check_if_films_exist_on_ant()

    dupe_checker = DupeChecker(films_to_dupe_check)
    films_checked_on_ant = dupe_checker.check_if_films_can_be_uploaded()

    write_film_list_to_csv(films_checked_on_ant, output_folder)

    logging.info("\nScript has ended")


if __name__ == "__main__":
    main()


import pandas as pd
import requests
import logging
from pathlib import Path
from typing import Any, Union
from ratelimit import limits, sleep_and_retry
import re
from requests.adapters import HTTPAdapter, Retry


class FilmSearcher:
    def __init__(self, film_list_df: pd.DataFrame, api_key: str):
        self.film_list_df: pd.DataFrame = film_list_df
        self.api_key: str = api_key
        self.ant_url = "https://anthelion.me/api.php"
        self.session: requests.Session = requests.Session()
        self.not_found_value: str = "NOT FOUND"

        retries = Retry(
            total=3, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504]
        )
        self.session.mount("https://", HTTPAdapter(max_retries=retries))

    def check_if_films_exist_on_ant(self) -> pd.DataFrame:
        """
        Given pandas DataFrame of film list, if list contains
        films that were on ANT and duplicates, then skip these films.
        For any not found, non-dupes, or new films in the list,
        search for these on ANT, indicating whether they exist on ANT or not.
        """
        regex_to_skip = r"^Duplicate|^Partial duplicate|is banned from ANT"

        self.film_list_df["Should skip"] = self.film_list_df[
            "Already on ANT?"
        ].str.contains(regex_to_skip, regex=True, na=False)

        films_to_skip = self.film_list_df.loc[self.film_list_df["Should skip"]]

        if not films_to_skip.empty:
            logging.info(
                "Skipping %s films already found on ANT in the previous output file...",
                len(films_to_skip),
            )
        else:
            logging.info(
                "No films were skipped as any existing film list did not contain duplicates."
            )

        films_to_process = (
            self.film_list_df.drop(films_to_skip.index)
            .sort_values(by="Parsed film title")
            .reset_index(drop=True)
        )
        films_to_process["API response"] = films_to_process["Parsed film title"].apply(
            self.check_if_film_exists_on_ant
        )

        films_to_dupe_check = (
            pd.concat([films_to_skip, films_to_process])
            .sort_values(by="Parsed film title")
            .reset_index(drop=True)
        )

        # Ensure NAs incl. those from old versions are converted to empty lists
        films_to_dupe_check["API response"] = (
            films_to_dupe_check["API response"].fillna("").apply(list)
        )

        return films_to_dupe_check

    def check_if_film_exists_on_ant(self, film_title: str) -> list[dict[str, Any]]:
        """
        Take a film title, and search for it using the ANT API.
        If an initial match is not found, re-search for a
        modified version of the film title if it meets certain conditions.
        """
        logging.info("\nSearching for %s...", film_title)
        title_checks = [
            (self.search_for_film_title_on_ant, ""),
            (self.search_for_film_if_contains_and, ""),
            (self.search_for_film_if_contains_potential_date_or_time, "time"),
            (self.search_for_film_if_contains_potential_date_or_time, "date"),
            (self.search_for_film_if_contains_aka, ""),
        ]
        for search_function, optional_argument in title_checks:
            try:
                if optional_argument:
                    search_result = search_function(film_title, optional_argument)
                else:
                    search_result = search_function(film_title)

                if search_result:
                    return search_result
            except Exception as err:
                logging.error(
                    "An unexpected error occured, skipping film:\n%s", str(err)
                )

        logging.info("--- Not found on ANT ---")
        return []

    def search_for_film_if_contains_and(self, film_title: str) -> list[dict[str, Any]]:
        """
        If film title contains and or &, replace with the oppposite
        and search for the new title on ANT. Else, return NOT FOUND.
        """
        and_word_regex = r"(?i)\sand\s"
        and_symbol_regex = r"(?i)\s&\s"

        if re.search(and_word_regex, film_title):
            logging.info("-- Film contains 'and'")
            return self.replace_word_and_re_search(film_title, and_word_regex, " & ")
        elif re.search(and_symbol_regex, film_title):
            logging.info("-- Film contains '&'")
            return self.replace_word_and_re_search(
                film_title, and_symbol_regex, " and "
            )

        return []

    def search_for_film_if_contains_potential_date_or_time(
        self, film_title: str, format: str
    ) -> list[dict[str, Any]]:
        """
        If film title 4 numbers e.g. 1208 East of Bucharest,
        add colon in middle, re-search title on ANT.
        Else, return NOT FOUND.
        """
        if format == "time":
            numbers_regex = r"(?<=\b\d\d)(?=\d\d\b)"
            replacement_value = ":"
        elif format == "date":
            numbers_regex = r"(?<=\b\d)(?=\d{1,2}\b)"
            replacement_value = "/"
        else:
            raise ValueError("The format argument must be 'time' or 'date'")

        if re.search(numbers_regex, film_title):
            logging.info(
                "-- Film title may contain a date or time without punctuation."
            )
            return self.replace_word_and_re_search(
                film_title, numbers_regex, replacement_value
            )

        return []

    def search_for_film_if_contains_aka(self, film_title: str) -> list[dict[str, Any]]:
        aka_regex = r"(?i)\saka\s"

        if re.search(aka_regex, film_title):
            logging.info("-- Film title may contain an alternate title")

            split_titles = re.split(aka_regex, film_title)

            for title in split_titles:
                logging.info("-- Searching for %s as well...", title)
                film_search = self.search_for_film_title_on_ant(title)
                if film_search != []:
                    return film_search
        return []

    def replace_word_and_re_search(
        self, film_title: str, regex_pattern: str, replacement: str
    ) -> list[dict[str, Any]]:
        cleaned_film_title = re.sub(
            regex_pattern,
            replacement,
            film_title,
        )
        logging.info("-- Searching for %s as well...", cleaned_film_title)

        return self.search_for_film_title_on_ant(cleaned_film_title)

    @sleep_and_retry
    @limits(calls=1, period=2)
    def search_for_film_title_on_ant(self, film_title: str) -> list[dict[str, Any]]:
        """
        Use the ANT API to search for a film title and
        return the first URL if found, else a NOT FOUND string
        """

        if not self.api_key:
            logging.error(
                "The API key entered is blank, please re-run and enter a valid API key"
            )
            raise SystemExit("Exiting due to blank API key")

        payload = {
            "api_key": self.api_key,
            "q": film_title,
            "t": "movie",
            "o": "json",
        }

        response = self.session.get(self.ant_url, params=payload)
        try:
            response.raise_for_status()
            response_json = response.json()
        except requests.exceptions.HTTPError as err:
            logging.error(
                "HTTP Error: %s",
                err,
            )
            if response.status_code == 429:
                logging.error(
                    "Try increasing the API search period limit to greater than 2"
                )
            elif response.status_code == 403:
                logging.error(
                    "Your API key may be invalid. Check the key in parameters.py"
                )
            raise SystemExit(err)
        except requests.exceptions.ConnectionError as err:
            logging.error("Connection error despite retries, exiting process\n")
            raise SystemExit(err)
        except requests.exceptions.Timeout as err:
            logging.error("Timeout error despite retires, exiting process\n")
            raise SystemExit(err)
        except requests.exceptions.RequestException as err:
            logging.error("The following error occured: %s", err)
            raise SystemExit(err)

        if response_json["response"]["total"] > 0:
            return response_json["item"]
        else:
            return []



from pathlib import Path
import logging
import pandas as pd


def write_film_list_to_csv(output_df: pd.DataFrame, output_folder: Path) -> None:
    if not output_df.empty:
        output_file_path = Path(output_folder).joinpath("Film list.csv")

        parent_directory = output_file_path.parents[0]
        if not parent_directory.is_dir():
            logging.info(
                "\nOutput directory '%s' not found but has been created",
                parent_directory,
            )
            parent_directory.mkdir(parents=True, exist_ok=True)

        logging.info("Writing list of films to %s...", output_file_path)

        output_df.to_csv(output_file_path, index=False, encoding="utf-8-sig")
    else:
        logging.warning("\nThe film list is empty, no file is being created.")
